@@ -1,27 +1,19 @@
 /*
  * Copyright (c) 2004 Jean-François Wauthy (pollux@xfce.org)
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,11 +24,6 @@
 #include <string.h>
 #endif
 
-#include <gtk/gtkversion.h>
-#if GTK_CHECK_VERSION(2, 4, 0)
-#undef GTK_DISABLE_DEPRECATED
-#endif
-
 /* */
 #include <libxfce4mcs/mcs-manager.h>
 #include <libxfcegui4/libxfcegui4.h>
@@ -59,9 +46,12 @@
 enum
 {
   COLUMN_NAME,
+  COLUMN_DESCRIPTION,
+  COLUMN_VERSION,
+  COLUMN_AUTHOR,
+  COLUMN_HOMEPAGE,
   COLUMN_PATH,
-  COLUMN_CLASS,
-  N_COLUMNS
+  N_COLUMNS,
 };
 
 /* setting */
@@ -70,10 +60,10 @@ static gchar *printing_system = NULL;
 /* */
 static GtkWidget *dialog = NULL;
 static GtkWidget *treeview;
-static GtkWidget *description;
-static GtkWidget *version;
-static GtkWidget *author;
-static GtkWidget *homepage;
+static GtkWidget *label_description;
+static GtkWidget *label_version;
+static GtkWidget *label_author;
+static GtkWidget *label_homepage;
 
 
 /* static prototypes */
@@ -125,8 +115,6 @@ mcs_plugin_init (McsPlugin * plugin)
   plugin->icon = xfce_themed_icon_load ("xfprint", 48);
   mcs_manager_notify (plugin->manager, CHANNEL);
 
-  save_settings (plugin);
-
   return (MCS_PLUGIN_INIT_OK);
 }
 
@@ -148,26 +136,10 @@ save_settings (McsPlugin * plugin)
 }
 
 /* Callbacks */
-static gboolean
-psinfos_free_foreach_func (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
-{
-  PrintingSystemInfo *psinfo;
-
-  gtk_tree_model_get (model, iter, COLUMN_CLASS, &psinfo, -1);
-  printing_system_info_free (psinfo);
-
-  return FALSE;
-}
-
 static gboolean
 cb_dialog_response (void)
 {
   if (dialog != NULL) {
-    GtkTreeModel *model;
-
-    /* free psclasses */
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
-    gtk_tree_model_foreach (model, psinfos_free_foreach_func, NULL);
     gtk_widget_destroy (dialog);
     dialog = NULL;
   }
@@ -185,46 +157,37 @@ cb_selection_changed (GtkTreeSelection * selection, gpointer data)
   plugin = (McsPlugin *) data;
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
-    PrintingSystemInfo *psinfo;
-    gchar *path;
+    gchar *description = NULL;
+    gchar *version = NULL;
+    gchar *author = NULL;
+    gchar *homepage = NULL;
+    gchar *path = NULL;
 
-    gtk_tree_model_get (model, &iter, COLUMN_CLASS, &psinfo, COLUMN_PATH, &path, -1);
+    gtk_tree_model_get (model, &iter, COLUMN_DESCRIPTION, &description,
+			COLUMN_VERSION, &version, COLUMN_AUTHOR, &author, COLUMN_HOMEPAGE, &homepage,
+			COLUMN_PATH, &path, -1);
 
-    if (psinfo) {
-      gtk_label_set_text (GTK_LABEL (description), psinfo->description);
-      gtk_label_set_text (GTK_LABEL (version), psinfo->version);
-      gtk_label_set_text (GTK_LABEL (author), psinfo->author);
-      gtk_label_set_text (GTK_LABEL (homepage), psinfo->homepage);
+    if (description)
+      gtk_label_set_text (GTK_LABEL (label_description), description);
+    if (version)
+      gtk_label_set_text (GTK_LABEL (label_version), version);
+    if (author)
+      gtk_label_set_text (GTK_LABEL (label_author), author);
+    if (homepage)
+      gtk_label_set_text (GTK_LABEL (label_homepage), homepage);
       
-      if (strcmp (psinfo->description, _("None")) == 0)
-	gtk_widget_set_sensitive (description, FALSE);
-      else
-	gtk_widget_set_sensitive (description, TRUE);
-      if (strcmp (psinfo->version, _("None")) == 0)
-	gtk_widget_set_sensitive (version, FALSE);
-      else
-	gtk_widget_set_sensitive (version, TRUE);
-      if (strcmp (psinfo->author, _("None")) == 0)
-	gtk_widget_set_sensitive (author, FALSE);
-      else
-	gtk_widget_set_sensitive (author, TRUE);
-      if (strcmp (psinfo->homepage, _("None")) == 0)
-	gtk_widget_set_sensitive (homepage, FALSE);
-      else
-	gtk_widget_set_sensitive (homepage, TRUE);
-      
-      g_free (printing_system);
-      printing_system = g_strdup (path);
-    } else {
-      g_free (printing_system);
-      printing_system = g_strdup ("none");
-    }
+    g_free (printing_system);
+    printing_system = g_strdup (path);
 
     mcs_manager_set_string (plugin->manager, "XfPrint/system", CHANNEL, printing_system);
     mcs_manager_notify (plugin->manager, CHANNEL);
     save_settings (plugin);
     
     g_free (path);
+    g_free (homepage);
+    g_free (author);
+    g_free (version);
+    g_free (description);
   }
 }
 
@@ -245,9 +208,9 @@ run_dialog (McsPlugin * plugin)
 
   GtkTreeModel *model;
   GtkTreeIter iter;
-  PrintingSystemInfo *psinfo;
 
   GDir *gdir;
+  GError *error = NULL;
   const char *file;
   GtkTreeSelection *selection;
 
@@ -279,7 +242,8 @@ run_dialog (McsPlugin * plugin)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin), GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start (GTK_BOX (hbox), swin, FALSE, FALSE, 0);
 
-  store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+  store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
+			      G_TYPE_STRING, G_TYPE_STRING);
   treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
   gtk_container_add (GTK_CONTAINER (swin), treeview);
@@ -307,11 +271,11 @@ run_dialog (McsPlugin * plugin)
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-  description = gtk_label_new ("");
-  gtk_label_set_line_wrap (GTK_LABEL (description), TRUE);
-  gtk_label_set_selectable (GTK_LABEL (description), TRUE);
-  gtk_misc_set_alignment (GTK_MISC (description), 0, 0);
-  gtk_table_attach (GTK_TABLE (table), description, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  label_description = gtk_label_new ("");
+  gtk_label_set_line_wrap (GTK_LABEL (label_description), TRUE);
+  gtk_label_set_selectable (GTK_LABEL (label_description), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (label_description), 0, 0);
+  gtk_table_attach (GTK_TABLE (table), label_description, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
   label_text = g_strdup_printf ("<b>%s</b>", _("Version"));
   label = gtk_label_new (label_text);
@@ -320,10 +284,10 @@ run_dialog (McsPlugin * plugin)
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 
-  version = gtk_label_new ("");
-  gtk_label_set_selectable (GTK_LABEL (version), TRUE);
-  gtk_misc_set_alignment (GTK_MISC (version), 0, 0);
-  gtk_table_attach (GTK_TABLE (table), version, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  label_version = gtk_label_new ("");
+  gtk_label_set_selectable (GTK_LABEL (label_version), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (label_version), 0, 0);
+  gtk_table_attach (GTK_TABLE (table), label_version, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
   label_text = g_strdup_printf ("<b>%s</b>", _("Author"));
   label = gtk_label_new (label_text);
@@ -332,10 +296,10 @@ run_dialog (McsPlugin * plugin)
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 
-  author = gtk_label_new ("");
-  gtk_label_set_selectable (GTK_LABEL (author), TRUE);
-  gtk_misc_set_alignment (GTK_MISC (author), 0, 0);
-  gtk_table_attach (GTK_TABLE (table), author, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  label_author = gtk_label_new ("");
+  gtk_label_set_selectable (GTK_LABEL (label_author), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (label_author), 0, 0);
+  gtk_table_attach (GTK_TABLE (table), label_author, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
   label_text = g_strdup_printf ("<b>%s</b>", _("Homepage"));
   label = gtk_label_new (label_text);
@@ -344,10 +308,10 @@ run_dialog (McsPlugin * plugin)
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 
-  homepage = gtk_label_new ("");
-  gtk_label_set_selectable (GTK_LABEL (homepage), TRUE);
-  gtk_misc_set_alignment (GTK_MISC (homepage), 0, 0);
-  gtk_table_attach (GTK_TABLE (table), homepage, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  label_homepage = gtk_label_new ("");
+  gtk_label_set_selectable (GTK_LABEL (label_homepage), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (label_homepage), 0, 0);
+  gtk_table_attach (GTK_TABLE (table), label_homepage, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
   /* handle selection */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -358,18 +322,16 @@ run_dialog (McsPlugin * plugin)
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
 
   /* Add the 'None' entry */
-  psinfo = g_new0 (PrintingSystemInfo, 1);
-  psinfo->name = g_strdup ("none");
-  psinfo->description = g_strdup (_("Use this backend if you just want to print to file."));
-  psinfo->version = g_strdup (VERSION);
-  psinfo->author = g_strdup ("Jean-François Wauthy");
-  psinfo->homepage = g_strdup ("http://www.xfce.org");
-
   gtk_list_store_append (GTK_LIST_STORE (model), &iter);
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-		      COLUMN_NAME, _("None"), COLUMN_PATH, "none", COLUMN_CLASS, psinfo, -1);
+		      COLUMN_NAME, _("None"), 
+		      COLUMN_DESCRIPTION, _("Use this backend if you just want to print to file."),
+		      COLUMN_VERSION, VERSION,
+		      COLUMN_AUTHOR, "Jean-François Wauthy",
+		      COLUMN_HOMEPAGE, "http://www.xfce.org",
+		      COLUMN_PATH, "none", -1);
   
-  gdir = g_dir_open (PLUGIN_DIR, 0, NULL);
+  gdir = g_dir_open (PLUGIN_DIR, 0, &error);
   if (gdir) {
     while ((file = g_dir_read_name (gdir))) {
       const char *s = file;
@@ -378,24 +340,25 @@ run_dialog (McsPlugin * plugin)
 
       if (strequal (s, SOEXT)) {
         char *path;
-        GModule *gm;
-        gpointer symbol;
+	PrintingSystem *ps = NULL;
 
         path = g_build_filename (PLUGIN_DIR, file, NULL);
 
-        gm = g_module_open (path, 0);
-
-        if (gm && g_module_symbol (gm, "printing_system_info_init_impl", &symbol)) {
-          void (*init) (PrintingSystemInfo * psinfo);
-
-          psinfo = g_new0 (PrintingSystemInfo, 1);
-
-          init = symbol;
-          init (psinfo);
+	/* can be optimized by using directly g_module_open and binding the required symbols
+	   instead of using printing_system_new 
+	*/
+	ps = printing_system_new (path);
 
+	if (ps) {
           gtk_list_store_append (GTK_LIST_STORE (model), &iter);
           gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-                              COLUMN_NAME, psinfo->name, COLUMN_PATH, file, COLUMN_CLASS, psinfo, -1);
+                              COLUMN_NAME, ps->name,
+			      COLUMN_DESCRIPTION, ps->description,
+			      COLUMN_VERSION, ps->version,
+			      COLUMN_AUTHOR, ps->author,
+			      COLUMN_HOMEPAGE, ps->homepage,
+			      COLUMN_PATH, file, -1);
+
           if (strcmp (file, printing_system) == 0) {
             GtkTreePath *treepath;
 
@@ -404,20 +367,19 @@ run_dialog (McsPlugin * plugin)
             gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), treepath, NULL, TRUE, 0.5, 0.0);
             gtk_tree_path_free (treepath);
           }
-
-          g_module_close (gm);
-        }
-        else if (gm) {
-          g_warning ("No 'printing_system_class_init_impl' function found in %s", path);
-          g_module_close (gm);
+	  g_object_unref (ps);
+        } else {
+          g_warning ("%s isn't a xfprint printing system module", path);
         }
 
         g_free (path);
       }
     }
-
+    g_dir_close (gdir);
+  } else {
+    g_warning ("An error occurred in the opening of the plugin directory (%d): %s", error->code, error->message);
+    g_error_free (error);
   }
-  g_dir_close (gdir);
 
   xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dialog));
   gtk_widget_show_all (dialog);
